@@ -226,6 +226,10 @@ pub enum ScreenContext {
     MoveFocusUp,
     MoveFocusRight,
     MoveFocusRightOrNextTab,
+    MovePaneDown,
+    MovePaneUp,
+    MovePaneRight,
+    MovePaneLeft,
     Exit,
     ScrollUp,
     ScrollUpAt,
@@ -244,7 +248,6 @@ pub enum ScreenContext {
     SetFixedHeight,
     SetFixedWidth,
     ClosePane,
-    ApplyLayout,
     NewTab,
     SwitchTabNext,
     SwitchTabPrev,
@@ -258,6 +261,8 @@ pub enum ScreenContext {
     MouseHold,
     Copy,
     ToggleTab,
+    AddClient,
+    RemoveClient,
 }
 
 /// Stack call representations corresponding to the different types of [`PtyInstruction`]s.
@@ -303,6 +308,7 @@ pub enum ServerContext {
     ClientExit,
     RemoveClient,
     Error,
+    KillSession,
     DetachSession,
     AttachClient,
 }
