@@ -1,6 +1,8 @@
 //! Definition of the actions that can be bound to keys.
 
 use super::command::RunCommandAction;
+use super::layout::TabLayout;
+use crate::input::options::OnForceClose;
 use serde::{Deserialize, Serialize};
 use zellij_tile::data::InputMode;
 
@@ -18,9 +20,9 @@ pub enum Direction {
 // As these actions are bound to the default config, please
 // do take care when refactoring - or renaming.
 // They might need to be adjusted in the default config
-// as well `../../../assets/config/default.yaml`
+// as well `../../assets/config/default.yaml`
 /// Actions that can be bound to keys.
-#[derive(Eq, Clone, Debug, PartialEq, Deserialize, Serialize)]
+#[derive(Clone, Debug, PartialEq, Deserialize, Serialize)]
 pub enum Action {
     /// Quit Zellij.
     Quit,
@@ -39,6 +41,7 @@ pub enum Action {
     /// Tries to move the focus pane in specified direction.
     /// If there is no pane in the direction, move to previous/next Tab.
     MoveFocusOrTab(Direction),
+    MovePane(Direction),
     /// Scroll up in focus pane.
     ScrollUp,
     /// Scroll up at point
@@ -47,12 +50,16 @@ pub enum Action {
     ScrollDown,
     /// Scroll down at point
     ScrollDownAt(Position),
+    /// Scroll down to bottom in focus pane.
+    ScrollToBottom,
     /// Scroll up one page in focus pane.
     PageScrollUp,
     /// Scroll down one page in focus pane.
     PageScrollDown,
     /// Toggle between fullscreen focus pane and normal layout.
     ToggleFocusFullscreen,
+    /// Toggle frames around panes in the UI
+    TogglePaneFrames,
     /// Toggle between sending text commands to all panes on the current tab and normal mode.
     ToggleActiveSyncTab,
     /// Open a new pane in the specified direction (relative to focus).
@@ -60,8 +67,8 @@ pub enum Action {
     NewPane(Option<Direction>),
     /// Close the focus pane.
     CloseFocus,
-    /// Create a new tab.
-    NewTab,
+    /// Create a new tab, optionally with a specified tab layout.
+    NewTab(Option<TabLayout>),
     /// Do nothing.
     NoOp,
     /// Go to the next tab.
@@ -71,6 +78,7 @@ pub enum Action {
     /// Close the current tab.
     CloseTab,
     GoToTab(u32),
+    ToggleTab,
     TabNameInput(Vec<u8>),
     /// Run speficied command in new pane.
     Run(RunCommandAction),
@@ -81,3 +89,12 @@ pub enum Action {
     MouseHold(Position),
     Copy,
 }
+
+impl From<OnForceClose> for Action {
+    fn from(ofc: OnForceClose) -> Action {
+        match ofc {
+            OnForceClose::Quit => Action::Quit,
+            OnForceClose::Detach => Action::Detach,
+        }
+    }
+}
